unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Variants, Controls, Forms,
  StdCtrls, Buttons, ImgList, Grids, ExtCtrls, iConfig, Menus;

type
  TConfigForm = class(TForm)
    Grid: TDrawGrid;
    Panel1: TPanel;
    ImageList1: TImageList;
    Timer1: TTimer;
    BtnNew: TBitBtn;
    BtnDel: TBitBtn;
    BtnUp: TBitBtn;
    BtnDwn: TBitBtn;
    PopupMenu: TPopupMenu;
    MenuAbout: TMenuItem;
    MenuConfig: TMenuItem;
    MenuExit: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure GridDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure GridMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure GridMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure GridSelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure GridGetEditText(Sender: TObject; ACol, ARow: Integer;
      var Value: String);
    procedure GridSetEditText(Sender: TObject; ACol, ARow: Integer;
      const Value: String);
    procedure BtnNewClick(Sender: TObject);
    procedure BtnDelClick(Sender: TObject);
    procedure BtnUpClick(Sender: TObject);
    procedure BtnDwnClick(Sender: TObject);
    procedure MenuExitClick(Sender: TObject);
    procedure MenuConfigClick(Sender: TObject);
    procedure MenuAboutClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    FCloseByApp: Boolean;
    procedure   ShowPopupMenu(Sender: TObject);
    procedure   ShowConfig(Sender: TObject);
  public

  end;

var
  ConfigForm: TConfigForm;

implementation

{$R *.DFM}

uses
    iTray, Info;

//  ---------------------------------------------------------------------------
//  `FbN{bNX`p
type
    TMouseStatus = record
        Downed: Boolean;
        Col: integer;
        Row: integer;
        Rect: TRect;
    end;
var
    CustomState: TMouseStatus = (Downed: false; Col: -1; Row: -1);

const
    CHECK_BTN_HEIGHT = 14;

//  ---------------------------------------------------------------------------
//  bZ[W
procedure   SimpleMessage(const s: string);
var
    Flag: integer;
begin
    Flag := MB_ICONINFORMATION + MB_OK;
    Application.MessageBox(PChar(s), PChar(Application.Title), Flag);
end;

//  ---------------------------------------------------------------------------
//  j[N
procedure   TConfigForm.ShowPopupMenu(Sender: TObject);
begin
    SetForegroundWindow(Handle);
    PopupMenu.Popup(Mouse.CursorPos.X, Mouse.CursorPos.Y);
end;

//  ---------------------------------------------------------------------------
//  j[N
procedure   TConfigForm.ShowConfig(Sender: TObject);
begin
    SetForegroundWindow(Handle);                  
    MenuConfig.OnClick(MenuConfig);
end;

//  ---------------------------------------------------------------------------
//  tH[
procedure TConfigForm.FormCreate(Sender: TObject);
begin
    Self.Caption := Application.Title + ' - ' + Self.Name;

    FCloseByApp := false;
    Application.ShowMainForm := false;

    SimpleTray := TSimpleTaskTray.Create(Application.Icon.Handle);
    SimpleTray.FOnLeftDblClick := ShowConfig;
    SimpleTray.FOnRightClick := ShowPopupMenu;
    SimpleTray.AddTray(ExtractFileName(ParamStr(0)));

    ConfList.LoadFromFile(ChangeFileExt(ParamStr(0), '.dat'));

    Grid.Canvas.Font.Assign(Grid.Font);
    Grid.DefaultColWidth := Grid.Canvas.TextWidth(' ');
    Grid.ColWidths[ 0] := Grid.Canvas.TextWidth('L ');
    Grid.ColWidths[ 1] := Grid.ColWidths[0];
    Grid.ColWidths[ 2] := Grid.Canvas.TextWidth('XX:XX ');
    Grid.ColWidths[10] := 200;
end;

//  ---------------------------------------------------------------------------
//  tH[j
procedure TConfigForm.FormDestroy(Sender: TObject);
begin
    ConfList.SaveToFile(ChangeFileExt(ParamStr(0), '.dat'));
    SimpleTray.Free();
end;

//  ---------------------------------------------------------------------------
//  qgύX
procedure TConfigForm.Timer1Timer(Sender: TObject);
var
    index: integer;
    i: integer;
    Temp: TIcon;
    Icon: HICON;
begin
    index := ConfList.DoYou();
    if (index <> -1) then
    begin
        Temp := TIcon.Create;
        for i := 0 to ImageList1.Count - 1 do
        begin
            ImageList1.GetIcon(i, Temp);
            Icon := Temp.Handle;
            SimpleTray.ModTray(Icon);
            Sleep(100);
        end;
        Temp.Free;
        SimpleTray.ModTray(Application.Icon.Handle);
        SimpleTray.ShowBalloon(
            ConfList.Get(index)^.Text,
            FormatDateTime('c', Now), bitINFO
        );
    end;
    //  \
    Grid.Repaint;
end;

//  ---------------------------------------------------------------------------
//  `
procedure TConfigForm.GridDrawCell(Sender: TObject; ACol, ARow: Integer;
  Rect: TRect; State: TGridDrawState);

    procedure   FillRect(const R: TRect; const BColor, FColor: TColor);
    begin
        Grid.Canvas.Brush.Color := BColor;
        Grid.Canvas.Font.Color := FColor;
        Grid.Canvas.FillRect(R);
    end;

const
    S_FIXED: array[0..10] of string
    = ('L', 'P', '', '', '', '', '', '', 'y', '', 'bZ[W');
var
    DfcState: Cardinal;
    RectH, RectW, MarginX, MarginY: integer;
    RCheck: TRect;
    BackColor, ForeColor: TColor;

    C: TConfig;
    CellText: string;
begin
    //  -----------------------------------------
    //  Œ
    if (gdFixed in State) then
    begin
        //  F
        BackColor := RGB(110,136,206);
        ForeColor := RGB(255,255,255);

        //  wihԂ
        FillRect(Rect, BackColor, ForeColor);
    end
    //  -----------------------------------------
    //  tH[JX̂Ƃ̔wiF
    else if (gdFocused in State) then
    begin
        //  IĂ
        if (gdSelected in State) then
        begin
            BackColor := RGB(140,166,236);
            ForeColor := RGB(255,255,255);
        end
        //  ȊO
        else
        begin
            BackColor := RGB(240,240,240);
            ForeColor := RGB(000,000,000);
        end;

        //  wihԂ
        FillRect(Rect, BackColor, ForeColor);
    end
    //  -----------------------------------------
    //  tH[JX̖Ƃ̔wiF
    else
    begin
        //  IĂ
        if (gdSelected in State) then
        begin
            BackColor := RGB(040,066,136);
            ForeColor := RGB(255,255,255);
        end
        //  ȊO
        else
        begin
            BackColor := RGB(255,255,255);
            ForeColor := RGB(000,000,000);
        end;

        //  wihԂ
        FillRect(Rect, BackColor, ForeColor);
    end;

    //  -----------------------------------------
    //  ͈͂̕E
    RectH := Rect.Bottom - Rect.Top;
    RectW := Rect.Right - Rect.Left;

    //  -----------------------------------------
    //  Œs
    if (ARow = 0) then
    begin
        //  ʒu
        CellText := S_FIXED[ACol];
        MarginX := (RectW - Grid.Canvas.TextWidth(CellText)) div 2;
        MarginY := (RectH - Grid.Canvas.TextHeight(CellText)) div 2;
        Inc(MarginX, Rect.Left);
        Inc(MarginY, Rect.Top);
        //  `
        Grid.Canvas.TextRect(Rect, MarginX, MarginY, CellText);
    end
    //  f[^s
    else if (ARow <= ConfList.Count) then
    begin
        //  o
        C := ConfList.Get(ARow - 1)^;

        //  
        if (ACol = 2) then
        begin
            CellText := Format('%2.2d:%2.2d', [C.Hour, C.Minute]);
            //  ʒu
            MarginX := (RectW - Grid.Canvas.TextWidth(CellText)) div 2;
            MarginY := (RectH - Grid.Canvas.TextHeight(CellText)) div 2;
            Inc(MarginX, Rect.Left);
            Inc(MarginY, Rect.Top);
            //  `
            Grid.Canvas.TextRect(Rect, MarginX, MarginY, CellText);
        end
        //  bZ[W
        else if (ACol = 10) then
        begin
            CellText := C.Text;
            //  ʒu
            MarginX := 5;
            MarginY := (RectH - Grid.Canvas.TextHeight(CellText)) div 2;
            Inc(MarginX, Rect.Left);
            Inc(MarginY, Rect.Top);
            //  `
            Grid.Canvas.TextRect(Rect, MarginX, MarginY, CellText);
        end
        //  `FbN{bNX
        else
        begin
            //  ʒu
            MarginX := (RectW - CHECK_BTN_HEIGHT) div 2;
            MarginY := (RectH - CHECK_BTN_HEIGHT) div 2;
            Inc(MarginX, Rect.Left);
            Inc(MarginY, Rect.Top);
            RCheck := Bounds(MarginX, MarginY, CHECK_BTN_HEIGHT, CHECK_BTN_HEIGHT);
            //  tO
            DfcState := DFCS_BUTTONCHECK + DFCS_FLAT;

            //  O
            case ACol of
                0:   if (C.Enable) then Inc(DfcState, DFCS_CHECKED);
                1:   if (C.AtOnce) then Inc(DfcState, DFCS_CHECKED);
                3..9:if (C.OnDay[ACol - 2]) then Inc(DfcState, DFCS_CHECKED);
            end;

            //  `
            if (CustomState.Downed)
               and (CustomState.Col = ACol) and (CustomState.Row = ARow) then
            begin
                Inc(DfcState, DFCS_INACTIVE);
            end;
            DrawFrameControl(Grid.Canvas.Handle, RCheck, DFC_BUTTON, DfcState);
        end;
    end;

    //  -----------------------------------------
    //  r`
    Grid.Canvas.MoveTo(Rect.Left, Rect.Bottom - 1);
    Grid.Canvas.LineTo(Rect.Right, Rect.Bottom - 1);
end;

//  ---------------------------------------------------------------------------
//  `FbN{bNXJn
procedure TConfigForm.GridMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
    Col, Row: integer;
    RCheck: TRect;
    RectH, RectW, MarginX, MarginY: integer;
begin
    //  󋵃tONA
    CustomState.Downed := false;

    //  ʒu𒲂ׂ
    Grid.MouseToCell(X, Y, Col, Row);
    CustomState.Col := Col;
    CustomState.Row := Row;

    //  Ă邩 - 1
    CustomState.Downed := (Col > -1) and (Col <> 2)
                      and (Row > 0) and (Row <= ConfList.Count);

    //  ̃Z͈̔
    CustomState.Rect := Grid.CellRect(Col, Row);

    if (CustomState.Downed) then
    begin
        //  ͈͂̕E
        RectW := CustomState.Rect.Right - CustomState.Rect.Left;
        RectH := CustomState.Rect.Bottom - CustomState.Rect.Top;
        //  `FbN{bNX̂ꏊ
        MarginX := (RectW - CHECK_BTN_HEIGHT) div 2;
        MarginY := (RectH - CHECK_BTN_HEIGHT) div 2;
        Inc(MarginX, CustomState.Rect.Left);
        Inc(MarginY, CustomState.Rect.Top);
        RCheck := Bounds(MarginX, MarginY, CHECK_BTN_HEIGHT, CHECK_BTN_HEIGHT);
        //  Ă邩 - 2
        CustomState.Downed := PtInRect(RCheck, Point(X, Y));
    end;

    //  ĕ`
    InvalidateRect(Grid.Handle, @CustomState.Rect, true);
end;

//  ---------------------------------------------------------------------------
//  `FbN{bNX슮
procedure TConfigForm.GridMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
    Col, Row: integer;
    P: PConfig;
begin
    //  Oʒuĕ`
    InvalidateRect(Grid.Handle, @CustomState.Rect, true);

    //  ݒ蒆
    if (CustomState.Downed) then
    begin
        //  ʒu𒲂ׂ
        Grid.MouseToCell(X, Y, Col, Row);

        //  ƓꏊȂ瑀
        if (CustomState.Col = Col) and (CustomState.Row = Row) then
        begin
            P := ConfList.Get(Row - 1);

            case Col of

            0: P^.Enable := not P^.Enable;
            1: P^.AtOnce := not P^.AtOnce;
            3..9: P^.OnDay[Col - 2] := not P^.OnDay[Col - 2];

            end;
        end;
    end;

    //  ̃Z͈̔
    CustomState.Rect := Grid.CellRect(Col, Row);

    //  ĕ`
    InvalidateRect(Grid.Handle, @CustomState.Rect, true);

    //  󋵃tONA
    CustomState.Downed := false;
end;

//  ---------------------------------------------------------------------------
//  ZI
procedure TConfigForm.GridSelectCell(Sender: TObject; ACol, ARow: Integer;
  var CanSelect: Boolean);
begin
    Grid.Options := Grid.Options - [goEditing]; 
    if (0 < ARow) and (ARow <= ConfList.Count) then
    begin
        if (ACol = 2) or (ACol = 10) then
            Grid.Options := Grid.Options + [goEditing];
    end
end;

//  ---------------------------------------------------------------------------
//  Jn
procedure TConfigForm.GridGetEditText(Sender: TObject; ACol, ARow: Integer;
  var Value: String);
var
    C: TConfig;
    HInpEdit: HWND;
    MaxLen: integer;
begin
    C := ConfList.Get(ARow - 1)^;
    MaxLen := 0;

    if (ACol = 2) then
    begin
        Value := Format('%2.2d:%2.2d', [C.Hour, C.Minute]);
        MaxLen := 5;
        ImeMode := imDisable;
        SetIme();
    end
    else if (ACol = 10) then
    begin
        Value := C.Text;
        MaxLen := 255;
        ImeMode := imHira;
        SetIme();
    end;

    if Grid.ControlCount > 0 then
    begin
        HInpEdit := TInplaceEdit(Grid.Controls[0]).Handle;
        PostMessage(HInpEdit, EM_LIMITTEXT, MaxLen, 0);
    end;
end;

//  ---------------------------------------------------------------------------
//  p
const
    MagicValue = '{F9FF3BE6-35EC-4BFB-A924-967749F3A43F}';
var
    PreviousEditorValue: string = MagicValue;

//  ---------------------------------------------------------------------------
//  
procedure TConfigForm.GridSetEditText(Sender: TObject; ACol, ARow: Integer;
  const Value: String);
var
    P: PConfig;
    V: TDateTime;
    H, M, S, MS: Word;
begin
    //  ҏW
    if Value <> PreviousEditorValue then
    begin
        PreviousEditorValue := Value;
        Exit;
    end;

    //  ҏWIɂ́AlxđĂ
    P := ConfList.Get(ARow - 1);
    if (ACol = 2) then
    begin
        TextToTime(Value, V);
        DecodeTime(V, H, M, S, MS);
        P^.Hour := H;
        P^.Minute := M;
    end
    else if (ACol = 10) then
    begin
        P^.Text := Value;
    end;

    //  ̂߂ɐ΂ɂ肦Ȃ
    PreviousEditorValue := MagicValue;
end;

//  ---------------------------------------------------------------------------
//  ǉ
procedure TConfigForm.BtnNewClick(Sender: TObject);
var
    C: TConfig;
begin
    FillChar(C, SizeOf(C), 0);
    ConfList.Add(C);
    Grid.RowCount := ConfList.Count + 1;
    Grid.Repaint();
end;

//  ---------------------------------------------------------------------------
//  폜
procedure TConfigForm.BtnDelClick(Sender: TObject);
begin
    if (0 < Grid.Row) and (Grid.Row <= ConfList.Count) then
    begin
        ConfList.Delete(Grid.Row - 1);
        Grid.RowCount := ConfList.Count + 1;
        Grid.Repaint();
    end;
end;

//  ---------------------------------------------------------------------------
//  Ɉړ
procedure TConfigForm.BtnUpClick(Sender: TObject);
begin
    if (1 < Grid.Row) and (Grid.Row <= ConfList.Count) then
    begin
        ConfList.Move(Grid.Row - 1, Grid.Row - 2);
        Grid.Row := Grid.Row - 1;
        Grid.Repaint();
    end;
end;

//  ---------------------------------------------------------------------------
//  Ɉړ
procedure TConfigForm.BtnDwnClick(Sender: TObject);
begin
    if (0 < Grid.Row) and (Grid.Row < ConfList.Count) then
    begin
        ConfList.Move(Grid.Row - 1, Grid.Row);
        Grid.Row := Grid.Row + 1;
        Grid.Repaint();
    end;
end;

procedure TConfigForm.MenuExitClick(Sender: TObject);
begin
    FCloseByApp := true;
    Self.Close();
end;

procedure TConfigForm.MenuConfigClick(Sender: TObject);
begin
    Self.Show();
end;

procedure TConfigForm.MenuAboutClick(Sender: TObject);
begin
    OpenInfoForm();
end;

procedure TConfigForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
    if (FCloseByApp) then
    begin
        CanClose := true;
    end
    else
    begin
        CanClose := false;
        Self.Hide();
    end;
end;

end.
