unit iConfig;

interface

uses
    Windows, SysUtils, DateUtils, Classes;

//  ---------------------------------------------------------------------------
//  ݒp\
type
    TConfig = packed record
        Enable: Boolean;
        Hour  : Byte;
        Minute: Byte;
        AtOnce: Boolean;
        OnDay : array[1..7] of Boolean;
        Text  : ShortString;
    end;
    PConfig = ^TConfig;

    TConfigList = class
    private
        FList: TList;
        function DoYouSub(const index, Hour, Minute, DayWeek: integer): Boolean;
    public
        constructor Create();
        destructor Destroy(); override;

        function ToString(const index: integer): string;
        procedure SaveToFile(const Path: string);
        procedure LoadFromFile(const Path: string);
        function DoYou(): integer;

        function Add(const Config: TConfig): integer;
        procedure Delete(const index: integer);
        function Get(const index: integer): PConfig;
        procedure Clear();
        procedure Move(const OldIndex, NewIndex: integer);

        function Count(): integer;
    end;

var
    ConfList: TConfigList = nil;

function TextToTime(const S: string; var Value: TDateTime): Boolean;

implementation

//  ---------------------------------------------------------------------------
//  ɕϊ
function TextToTime(const S: string; var Value: TDateTime): Boolean;
begin
    Result := TryStrToTime(S, Value);
    if (not Result) then Value := 0;
end;

//  ---------------------------------------------------------------------------
//  RXgN^
constructor TConfigList.Create();
begin
    FList := TList.Create;
end;

//  ---------------------------------------------------------------------------
//  fXgN^
destructor TConfigList.Destroy();
begin
    Clear();
    FList.Free;
end;

//  ---------------------------------------------------------------------------
//  f[^
function TConfigList.Count(): integer;
begin
    Result := FList.Count;
end;

//  ---------------------------------------------------------------------------
//  ɕϊ
function TConfigList.ToString(const index: integer): string;
const
    DAY_OF_WEEK: array[1..7] of string = ('', '', '', '', '', 'y', '');
var
    s, s1: string;
    P: PConfig;
    i: integer;
begin
    Result := '';

    P := Get(index);

    if P^.Enable then s := 'L' else s := '';
    Result := Result + s + ' ';

    s := Format('%2.2d:%2.2d', [P^.Hour, P^.Minute]);
    Result := Result + s + ' ';

    if P^.AtOnce then s := 'P' else s := 'J';
    Result := Result + s + ' ';

    s := '';
    for i := Low(P^.OnDay) to High(P^.OnDay) do
    begin
        if (P^.OnDay[i]) then s1 := DAY_OF_WEEK[i] else s1 := '@';
        s := s + s1;
    end;
    Result := Result + s + ' ' + P^.Text;
end;

//  ---------------------------------------------------------------------------
//  ǉ
function TConfigList.Add(const Config: TConfig): integer;
var
    P: PConfig;    
begin
    new(P);
    P^ := Config;
    Result := FList.Add(P);
end;

//  ---------------------------------------------------------------------------
//  폜
procedure TConfigList.Delete(const index: integer);
begin
    Dispose(Get(index));
    FList.Delete(index);
end;

//  ---------------------------------------------------------------------------
//  擾
function TConfigList.Get(const index: integer): PConfig;
begin
    Result := PConfig(FList.Items[index]);
end;

//  ---------------------------------------------------------------------------
//  NA
procedure TConfigList.Clear();
begin
    while Count > 0 do Delete(0);
end;

//  ---------------------------------------------------------------------------
//  ړ
procedure TConfigList.Move(const OldIndex, NewIndex: integer);
begin
    FList.Move(OldIndex, NewIndex);
end;

//  ---------------------------------------------------------------------------
//  Ă
procedure TConfigList.SaveToFile(const Path: string);
var
    F: file of TConfig;
    i: integer;
begin
    AssignFile(F, Path);
    Rewrite(F);
    try
        for i := 0 to Count - 1 do
        begin
            Write(F, Get(i)^);
        end;
    finally
        CloseFile(F);
    end;
end;

//  ---------------------------------------------------------------------------
//  ǂł
procedure TConfigList.LoadFromFile(const Path: string);
var
    F: file of TConfig;
    Work: TConfig;
begin
    if FileExists(Path) then
    begin
        AssignFile(F, Path);
        Reset(F);
        try
            while not Eof(F) do
            begin
                Read(F, Work);
                Add(Work);
            end;
        finally
            CloseFile(F);
        end;
    end;
end;

//  ---------------------------------------------------------------------------
//  Ă - sub
function TConfigList.DoYouSub(const index, Hour, Minute, DayWeek: integer): Boolean;
var
    P: PConfig;
begin
    P := Get(index);

    Result := false;
    //  Ȃʖ
    if (not P^.Enable) then exit;
    //  ԂႤȂʖ
    if (Hour <> P^.Hour) then exit;
    //  ႤȂʖ
    if (Minute <> P^.Minute) then exit;
    //  jȂʖ
    if (not P^.OnDay[DayWeek]) then exit;
    //  s܂
    Result := true;
    //  Ȃ
    if (P^.AtOnce) then P^.Enable := false;
end;

//  ---------------------------------------------------------------------------
//  Ă - main
function TConfigList.DoYou(): integer;
var
    i, DayWeek: integer;
    DateNow: TDateTime;
    H, M, S, MS: Word;
begin
    Result := -1;

    DateNow := Now();
    DecodeTime(DateNow, H, M, S, MS);
    DayWeek := DayOfTheWeek(DateNow);

    for i := 0 to FList.Count - 1 do
    begin
        if (DoYouSub(i, H, M, DayWeek)) then
        begin
            Result := i;
            exit;
        end;
    end;
end;

//  ---------------------------------------------------------------------------
initialization
    ConfList := TConfigList.Create;
//  ---------------------------------------------------------------------------
finalization
    ConfList.Free;
//  ---------------------------------------------------------------------------
end.

